import string
from datetime import timedelta
from threading import Thread

from vk_api.longpoll import VkLongPoll, VkEventType
from yoomoney import Client, Quickpay

from qiwip2py import QiwiP2P
from scr.questions import QUESTIONS_STEPS
from scr.settings import *
from scr.vk_functions import *

users = {}

if YOOMONEY_TOKEN:
    client = Client(YOOMONEY_TOKEN)
    user = client.account_info()

    print("Account number:", user.account)
    print("Account balance:", user.balance)
    print("Account currency code in ISO 4217 format:", user.currency)
    print("Account status:", user.account_status)
    print("Account type:", user.account_type)
    print("Extended balance information:")
    for pair in vars(user.balance_details):
        print("\t-->", pair, ":", vars(user.balance_details).get(pair))
    print("Information about linked bank cards:")

    cards = user.cards_linked
    if len(cards) != 0:
        for card in cards:
            print(card.pan_fragment, " - ", card.type)
    else:
        print("No card is linked to the account")

if QIWI_SECRET_KEY:
    qiwi_p2p = QiwiP2P(secret_key=QIWI_SECRET_KEY)


def thread(my_func):
    def wrapper(*args, **kwargs):
        my_thread = Thread(target=my_func, args=args, kwargs=kwargs)
        my_thread.start()

    return wrapper


def gen_bill_id():
    return ''.join(random.sample(f"{string.digits}{string.ascii_letters}{'-'}" * 63, 10))


def send_by_step(person_id, step):
    attachment = ''
    if QUESTIONS_STEPS[step].get('q_file'):
        f_path = QUESTIONS_STEPS[step].get('q_file')
        for f_path in f_path.split(','):
            attachment += upload_vk_doc(f_path, person_id, f_path[f_path.rfind('/') + 1:]) + ','
    if QUESTIONS_STEPS[step].get('q_attachment'):
        attachment += QUESTIONS_STEPS[step].get('q_attachment')
    msg_keyboard(QUESTIONS_STEPS[step]['q_text'], person_id,
                 QUESTIONS_STEPS[step]['buttons'], attachment=attachment)


def qiwi_pay(body, person_id, user_step):
    if not QIWI_SECRET_KEY:
        msg('Метод оплаты пока что недоступен', person_id)
        users[person_id]['step'] = users[person_id]['last_step']
        if users[person_id]['last_step'] == 'step_chose_a':
            users[person_id]['last_step'] = 'step6'
        else:
            users[person_id]['last_step'] = 'step7'
    if body == 'QIWI':
        price = STEP7_PRICE if users[person_id]['step'] == 'step8_qiwi' else STEP6_PRICE
        bill = qiwi_p2p.create_bill(bill_id=gen_bill_id(), amount=price,
                                    expiration_datetime=timedelta(minutes=20))
        users[person_id]['bill_id_qiwi'] = bill.bill_id
        keyboard_buttons = {"one_time": False, "buttons": []}
        keyboard_buttons['buttons'].append(
            [{"action": {"type": "open_link", "label": 'Оплатить', 'link': bill.pay_url}}])
        keyboard_buttons['buttons'].append(create_button('Проверить', 'primary'))
        keyboard_buttons['buttons'].append(create_button('Отмена', 'negative'))
        msg_keyboard(f'Оплата через QIWI\nСумма: {price}₽\nСсылка для оплаты:\n{bill.pay_url}', person_id,
                     keyboard=json.dumps(keyboard_buttons))
    elif body == 'Проверить':
        bill = qiwi_p2p.get_bill(users[person_id]['bill_id_qiwi'])
        print(bill)
        print(bill.status['value'])
        if bill.status['value'] == 'PAID':
            print(person_id, 'Оплата прошла успешно!')
            users[person_id]['step'] = 'step8a' if user_step == 'step8a_qiwi' else 'step8'
            send_by_step(person_id, users[person_id]['step'])
            return 0
        else:
            msg('Вы не оплатили счет!', person_id)
            return 0
    elif body == 'Отмена':
        qiwi_p2p.reject_bill('test')
        print(users[person_id])
        users[person_id]['step'] = users[person_id]['last_step']
        if users[person_id]['last_step'] == 'step_chose_a':
            users[person_id]['last_step'] = 'step6'
        else:
            users[person_id]['last_step'] = 'step7'
        send_by_step(person_id, users[person_id]['step'])


def yoomoney_pay(body, person_id, user_step):
    if not YOOMONEY_TOKEN:
        msg('Метод оплаты пока что недоступен', person_id)
        users[person_id]['step'] = users[person_id]['last_step']
        if users[person_id]['last_step'] == 'step_chose_a':
            users[person_id]['last_step'] = 'step6'
        else:
            users[person_id]['last_step'] = 'step7'
        return 0
    if body == 'ЮКасса':
        price = STEP7_PRICE if users[person_id]['step'] == 'step8_yoomoney' else STEP6_PRICE
        bill_id = gen_bill_id()
        quickpay = Quickpay(
            receiver=user.account,
            quickpay_form="shop",
            targets="Оплата инструкции",
            paymentType="SB",
            sum=price,
            label=bill_id
        )
        print(quickpay)
        print(quickpay.base_url)
        print(quickpay.redirected_url)
        users[person_id]['bill_id_yoomoney'] = bill_id
        keyboard_buttons = {"one_time": False, "buttons": []}
        keyboard_buttons['buttons'].append(
            [{"action": {"type": "open_link", "label": 'Оплатить', 'link': quickpay.base_url}}])
        keyboard_buttons['buttons'].append(create_button('Проверить', 'primary'))
        keyboard_buttons['buttons'].append(create_button('Отмена', 'negative'))
        msg_keyboard(f'Оплата через ЮКасса\nСумма: {price}₽\nСсылка для оплаты:\n{quickpay.base_url}', person_id,
                     keyboard=json.dumps(keyboard_buttons))
    elif body == 'Проверить':
        history = client.operation_history(label=users[person_id]['bill_id_yoomoney'])
        print(history.operations)
        for operation in history.operations:
            print("Operation:", operation.operation_id)
            print("\tStatus     -->", operation.status)
            print("\tDatetime   -->", operation.datetime)
            print("\tTitle      -->", operation.title)
            print("\tPattern id -->", operation.pattern_id)
            print("\tDirection  -->", operation.direction)
            print("\tAmount     -->", operation.amount)
            print("\tLabel      -->", operation.label)
            print("\tType       -->", operation.type)
            if operation.status == 'success' and operation.label == users[person_id]['bill_id_yoomoney']:
                print(person_id, 'Оплата прошла успешно!')
                users[person_id]['step'] = 'step8a' if user_step == 'step8a_qiwi' else 'step8'
                send_by_step(person_id, users[person_id]['step'])
                return 0
            else:
                msg('Вы не оплатили счет!', person_id)
                return 0
        msg('Вы не оплатили счет!', person_id)
    elif body == 'Отмена':
        # qiwi_p2p.reject_bill('test')
        print(users[person_id])
        users[person_id]['step'] = users[person_id]['last_step']
        if users[person_id]['last_step'] == 'step_chose_a':
            users[person_id]['last_step'] = 'step6'
        else:
            users[person_id]['last_step'] = 'step7'
        send_by_step(person_id, users[person_id]['step'])


@thread
def check_message(person_id, body):
    print("%s: %s" % (person_id, body))
    try:
        if person_id not in users:
            send_by_step(person_id, 'start')
            users[person_id] = {}
            users[person_id]['step'] = 'start'
            return 0
        user_step = users[person_id]['step']
        print(users[person_id])
        if user_step in ['step8', 'step8a']:
            print('Пользователь уже купил подписку')
            if body == "Контакты":
                send_by_step(person_id, 'step_contacts_after_buy')
            elif body == "Вопрос-Ответ":
                send_by_step(person_id, 'step_questions_after_buy')
            else:
                msg('Спасибо за покупку', person_id)
            return 0
        if 'step8' in user_step or 'step_chose' in user_step:
            print('Покупка инструкции')
            new_step = None
            if user_step in QUESTIONS_STEPS and body in QUESTIONS_STEPS[user_step]['answers']:
                new_step = QUESTIONS_STEPS[user_step]['answers'][body]
            if new_step:
                if new_step == 'step_back':
                    users[person_id]['step'] = users[person_id]['last_step']
                    users[person_id]['last_step'] = users[person_id]['last_step']
                    send_by_step(person_id, users[person_id]['step'])
                    return 1
                users[person_id]['last_step'] = users[person_id]['step']
                users[person_id]['step'] = new_step
            if 'qiwi' in users[person_id]['step']:
                qiwi_pay(body, person_id, user_step)
            elif 'yoomoney' in users[person_id]['step']:
                yoomoney_pay(body, person_id, user_step)
            elif 'vk_pay' in users[person_id]['step']:
                pass
        elif user_step not in QUESTIONS_STEPS:
            users.pop(person_id)
            return 0
        elif body in QUESTIONS_STEPS[user_step]['answers']:
            new_step = QUESTIONS_STEPS[user_step]['answers'][body]
            print(new_step)
            if new_step == 'step_back':
                new_step = users[person_id]['last_step']
            send_by_step(person_id, new_step)
            if user_step not in ['step10', 'step_back', 'step_contacts', 'step_questions']:
                users[person_id]['last_step'] = user_step
            users[person_id]['step'] = new_step

        elif 'step8' in user_step or 'step_chose' in user_step:
            msg('Не понимаю вас', person_id)
        elif body == "Контакты":
            send_by_step(person_id, 'step_contacts')
            users[person_id]['step'] = 'step_contacts'
            users[person_id]['last_step'] = user_step
        elif body == "Вопрос-Ответ":
            send_by_step(person_id, 'step_questions')
            users[person_id]['step'] = 'step_questions'
            users[person_id]['last_step'] = user_step
        else:
            msg('Не понимаю вас', person_id)
    except Exception as E:
        print('[ERROR] Ошибка обработки сообщения', E)


def main():
    longpoll = VkLongPoll(vk)
    for event in longpoll.listen():
        try:
            if event.type == VkEventType.MESSAGE_NEW:
                if event.to_me:
                    try:
                        msg = vk_.messages.getById(message_ids=event.message_id)
                        if msg:
                            for item in msg['items']:
                                attachments = item['attachments']
                                try:
                                    attachments = attachments[0]
                                except:
                                    pass
                                if attachments:
                                    print('Attachments:', f"{attachments['type']}{attachments[attachments['type']]['owner_id']}_"
                                          f"{attachments[attachments['type']]['id']}_{attachments[attachments['type']]['access_key']}")
                    except Exception as E:
                        print(E)
                    person_id = event.user_id
                    body = event.text
                    check_message(person_id, body)
        except Exception as E:
            print('[ERROR]', E)


while True:
    try:
        main()
    except Exception as E:
        print('[ERROR] main: ', E)
