import json
import random

import requests
from vk_api import vk_api

from .settings import VK_TOKEN

vk = vk_api.VkApi(token=VK_TOKEN)
vk_ = vk.get_api()


def create_button(text, color):
    return [{"action": {"type": "text", "label": text}, "color": color}]


def create_button_link(text, link):
    return [{"action": {"type": "open_link", "label": text, 'link': link}}]


def create_keyboard(texts, questions=True):
    keyboard_buttons = {"one_time": False, "buttons": []}
    for text in texts:
        if isinstance(text, str):
            keyboard_buttons["buttons"].append(create_button(text, "secondary"))
        elif isinstance(text, list):
            keyboard_buttons["buttons"].append(create_button(text[0], text[1]))
        elif isinstance(text, tuple):
            keyboard_buttons["buttons"].append(create_button_link(text[0], text[1]))
    if questions:
        keyboard_buttons["buttons"].append(create_button('Контакты', "positive"))
        keyboard_buttons["buttons"].append(create_button('Вопрос-Ответ', "positive"))
    keyboard_json = json.dumps(keyboard_buttons)
    return keyboard_json


def msg_keyboard(text, peer_id, keyboard, attachment=None):
    vk.method("messages.send",
              {"peer_id": peer_id, "message": text, "attachment": attachment, "keyboard": keyboard,
               "random_id": random.randint(1, 2147483647)})


def msg(text, peer_id, attachment=None):
    vk.method("messages.send", {"peer_id": peer_id, "attachment": attachment, "message": text,
                                "random_id": random.randint(1, 2147483647)})

def upload_vk_doc(file_path, user_id, title='Инструкция'):
    upload_url = vk_.docs.getMessagesUploadServer(type='doc', peer_id=user_id)['upload_url']
    response = requests.post(upload_url, files={'file': open(file_path, 'rb')})
    result = json.loads(response.text)
    file = result['file']
    file_info = vk_.docs.save(file=file, title=title, tags=[])
    owner_id = file_info['doc']['owner_id']
    photo_id = file_info['doc']['id']
    return f"doc{owner_id}_{photo_id}"